@@ -14,7 +14,9 @@
 #include <KLocalizedString> // i18n
 #include <KPluginFactory>
 #include <KSharedConfig>
-#include <ktexteditor/view.h>
+
+#include <KTextEditor/Cursor>
+#include <KTextEditor/View>
 
 #include <QClipboard>
 #include <QDialogButtonBox>
@@ -105,7 +107,6 @@ KTextEditor::ConfigPage *KateBtBrowserPlugin::configPage(int number, QWidget *pa
 
 KateBtBrowserPluginView::KateBtBrowserPluginView(KateBtBrowserPlugin *plugin, KTextEditor::MainWindow *mainWindow)
     : QObject(mainWindow)
-    , m_plugin(plugin)
 {
     // init console
     QWidget *toolview = mainWindow->createToolView(plugin,
@@ -376,5 +377,6 @@ KateBtConfigDialog::~KateBtConfigDialog()
 }
 
 #include "katebacktracebrowser.moc"
+#include "moc_katebacktracebrowser.cpp"
 
 // kate: space-indent on; indent-width 4; replace-tabs on;
