@@ -9,6 +9,7 @@
 
 #include "plugin_kategdb.h"
 
+#include <QBoxLayout>
 #include <QFile>
 #include <QFontDatabase>
 #include <QKeyEvent>
@@ -29,29 +30,69 @@
 #include <KLocalizedString>
 #include <KPluginFactory>
 
+#include "debugconfigpage.h"
+#include <QDir>
 #include <ktexteditor/document.h>
 #include <ktexteditor/editor.h>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <ktexteditor/markinterface.h>
+#endif
 #include <ktexteditor/view.h>
 
 K_PLUGIN_FACTORY_WITH_JSON(KatePluginGDBFactory, "kategdbplugin.json", registerPlugin<KatePluginGDB>();)
 
+static const QString CONFIG_DEBUGPLUGIN{QStringLiteral("debugplugin")};
+static const QString CONFIG_DAP_CONFIG{QStringLiteral("DAPConfiguration")};
+
 KatePluginGDB::KatePluginGDB(QObject *parent, const VariantList &)
     : KTextEditor::Plugin(parent)
+    , m_settingsPath(QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation) + QStringLiteral("/debugger"))
+    , m_defaultConfigPath(QUrl::fromLocalFile(m_settingsPath + QStringLiteral("/dap.json")))
 {
     // FIXME KF5 KGlobal::locale()->insertCatalog("kategdbplugin");
+    QDir().mkpath(m_settingsPath);
+    readConfig();
+}
+
+int KatePluginGDB::configPages() const
+{
+    return 1;
+}
+KTextEditor::ConfigPage *KatePluginGDB::configPage(int number, QWidget *parent)
+{
+    if (number != 0) {
+        return nullptr;
+    }
+
+    return new DebugConfigPage(parent, this);
 }
 
 KatePluginGDB::~KatePluginGDB()
 {
 }
 
+void KatePluginGDB::readConfig()
+{
+    KConfigGroup config(KSharedConfig::openConfig(), CONFIG_DEBUGPLUGIN);
+    m_configPath = config.readEntry(CONFIG_DAP_CONFIG, QUrl());
+
+    Q_EMIT update();
+}
+
+void KatePluginGDB::writeConfig() const
+{
+    KConfigGroup config(KSharedConfig::openConfig(), CONFIG_DEBUGPLUGIN);
+    config.writeEntry(CONFIG_DAP_CONFIG, m_configPath);
+
+    Q_EMIT update();
+}
+
 QObject *KatePluginGDB::createView(KTextEditor::MainWindow *mainWindow)
 {
     return new KatePluginGDBView(this, mainWindow);
 }
 
-KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::MainWindow *mainWin)
+KatePluginGDBView::KatePluginGDBView(KatePluginGDB *plugin, KTextEditor::MainWindow *mainWin)
     : QObject(mainWin)
     , m_mainWin(mainWin)
 {
@@ -62,7 +103,7 @@ KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::M
     m_focusOnInput = true;
     m_activeThread = -1;
 
-    KXMLGUIClient::setComponentName(QStringLiteral("kategdb"), i18n("Kate GDB"));
+    KXMLGUIClient::setComponentName(QStringLiteral("kategdb"), i18n("Kate Debug"));
     setXMLFile(QStringLiteral("ui.rc"));
 
     m_toolView.reset(
@@ -75,6 +116,13 @@ KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::M
                                                           i18n("Locals and Stack")));
 
     m_tabWidget = new QTabWidget(m_toolView.get());
+
+    // buttons widget, buttons are initialized at action connections
+    m_buttonWidget = new QWidget(m_tabWidget);
+    auto buttonsLayout = new QHBoxLayout(m_buttonWidget);
+    buttonsLayout->setContentsMargins(0, 1, 0, 3);
+    m_tabWidget->setCornerWidget(m_buttonWidget);
+
     // Output
     m_outputArea = new QTextEdit();
     m_outputArea->setAcceptRichText(false);
@@ -144,12 +192,12 @@ KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::M
     locStackSplitter->setOrientation(Qt::Vertical);
 
     // config page
-    m_configView = new ConfigView(nullptr, mainWin);
+    m_configView = new ConfigView(nullptr, mainWin, plugin);
 
     m_ioView = std::make_unique<IOView>();
     connect(m_configView, &ConfigView::showIO, this, &KatePluginGDBView::showIO);
 
-    m_tabWidget->addTab(m_gdbPage, i18nc("Tab label", "GDB Output"));
+    m_tabWidget->addTab(m_gdbPage, i18nc("Tab label", "Debug Output"));
     m_tabWidget->addTab(m_configView, i18nc("Tab label", "Settings"));
 
     m_debugView = new Backend(this);
@@ -221,18 +269,23 @@ KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::M
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-run")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::Key_F7)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotDebug);
+    // This button is handled differently due to it having multiple actions
+    m_continueButton = new QToolButton();
+    m_continueButton->setAutoRaise(true);
+    buttonsLayout->addWidget(m_continueButton);
 
     a = actionCollection()->addAction(QStringLiteral("kill"));
     a->setText(i18n("Kill / Stop Debugging"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-stop")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F7)));
     connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotKill);
+    buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("rerun"));
-    a->setText(i18n("Restart Debugging"));
-    a->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
-    connect(a, &QAction::triggered, this, &KatePluginGDBView::slotRestart);
+    a = actionCollection()->addAction(QStringLiteral("continue"));
+    a->setText(i18n("Continue"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
+    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::Key_F7)));
+    connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotContinue);
 
     a = actionCollection()->addAction(QStringLiteral("toggle_breakpoint"));
     a->setText(i18n("Toggle Breakpoint / Break"));
@@ -245,35 +298,41 @@ KatePluginGDBView::KatePluginGDBView(KTextEditor::Plugin *plugin, KTextEditor::M
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-into")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::Key_F10)));
     connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotStepInto);
+    buttonsLayout->addWidget(createDebugButton(a));
 
     a = actionCollection()->addAction(QStringLiteral("step_over"));
     a->setText(i18n("Step Over"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-over")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::Key_F9)));
     connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotStepOver);
+    buttonsLayout->addWidget(createDebugButton(a));
 
     a = actionCollection()->addAction(QStringLiteral("step_out"));
     a->setText(i18n("Step Out"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-step-out")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F10)));
     connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotStepOut);
-
-    a = actionCollection()->addAction(QStringLiteral("move_pc"));
-    a->setText(i18nc("Move Program Counter (next execution)", "Move PC"));
-    connect(a, &QAction::triggered, this, &KatePluginGDBView::slotMovePC);
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F9)));
+    buttonsLayout->addWidget(createDebugButton(a));
 
     a = actionCollection()->addAction(QStringLiteral("run_to_cursor"));
     a->setText(i18n("Run To Cursor"));
     a->setIcon(QIcon::fromTheme(QStringLiteral("debug-run-cursor")));
     actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::SHIFT | Qt::Key_F9)));
     connect(a, &QAction::triggered, this, &KatePluginGDBView::slotRunToCursor);
+    buttonsLayout->addWidget(createDebugButton(a));
 
-    a = actionCollection()->addAction(QStringLiteral("continue"));
-    a->setText(i18n("Continue"));
-    a->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
-    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::Key_F7)));
-    connect(a, &QAction::triggered, m_debugView, &DebugViewInterface::slotContinue);
+    a = actionCollection()->addAction(QStringLiteral("rerun"));
+    a->setText(i18n("Restart Debugging"));
+    a->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
+    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::CTRL | Qt::SHIFT | Qt::Key_F7)));
+    connect(a, &QAction::triggered, this, &KatePluginGDBView::slotRestart);
+    buttonsLayout->addWidget(createDebugButton(a));
+    buttonsLayout->addStretch();
+
+    a = actionCollection()->addAction(QStringLiteral("move_pc"));
+    a->setText(i18nc("Move Program Counter (next execution)", "Move PC"));
+    connect(a, &QAction::triggered, this, &KatePluginGDBView::slotMovePC);
+    actionCollection()->setDefaultShortcut(a, QKeySequence((Qt::ALT | Qt::Key_F9)));
 
     a = actionCollection()->addAction(QStringLiteral("print_value"));
     a->setText(i18n("Print Value"));
@@ -404,20 +463,32 @@ void KatePluginGDBView::slotToggleBreakpoint()
 
 void KatePluginGDBView::slotBreakpointSet(const QUrl &file, int line)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if (auto doc = m_kateApplication->findUrl(file)) {
+        doc->addMark(line, KTextEditor::Document::BreakpointActive);
+    }
+#else
     KTextEditor::MarkInterfaceV2 *iface = qobject_cast<KTextEditor::MarkInterfaceV2 *>(m_kateApplication->findUrl(file));
 
     if (iface) {
         iface->addMark(line, KTextEditor::MarkInterface::BreakpointActive);
     }
+#endif
 }
 
 void KatePluginGDBView::slotBreakpointCleared(const QUrl &file, int line)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if (auto doc = m_kateApplication->findUrl(file)) {
+        doc->removeMark(line, KTextEditor::Document::BreakpointActive);
+    }
+#else
     KTextEditor::MarkInterface *iface = qobject_cast<KTextEditor::MarkInterface *>(m_kateApplication->findUrl(file));
 
     if (iface) {
         iface->removeMark(line, KTextEditor::MarkInterface::BreakpointActive);
     }
+#endif
 }
 
 void KatePluginGDBView::slotMovePC()
@@ -445,11 +516,17 @@ void KatePluginGDBView::slotGoTo(const QUrl &url, int lineNum)
     if ((url == m_lastExecUrl) && (lineNum == m_lastExecLine)) {
         return;
     } else {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        if (auto doc = m_kateApplication->findUrl(m_lastExecUrl)) {
+            doc->removeMark(m_lastExecLine, KTextEditor::Document::Execution);
+        }
+#else
         KTextEditor::MarkInterfaceV2 *iface = qobject_cast<KTextEditor::MarkInterfaceV2 *>(m_kateApplication->findUrl(m_lastExecUrl));
 
         if (iface) {
             iface->removeMark(m_lastExecLine, KTextEditor::MarkInterface::Execution);
         }
+#endif
     }
 
     // skip not existing files
@@ -484,6 +561,11 @@ void KatePluginGDBView::enableDebugActions(bool enable)
     actionCollection()->action(QStringLiteral("kill"))->setEnabled(m_debugView->debuggerRunning());
     actionCollection()->action(QStringLiteral("rerun"))->setEnabled(m_debugView->debuggerRunning());
 
+    // Combine start and continue button to same button
+    m_continueButton->removeAction(actionCollection()->action(!enable ? QStringLiteral("continue") : QStringLiteral("debug")));
+    m_continueButton->setDefaultAction(actionCollection()->action(enable ? QStringLiteral("continue") : QStringLiteral("debug")));
+    m_continueButton->setIcon(QIcon::fromTheme(QStringLiteral("media-playback-start")));
+
     m_inputArea->setEnabled(enable && !m_debugView->debuggerBusy());
     m_threadCombo->setEnabled(enable);
     m_stackTree->setEnabled(enable);
@@ -510,6 +592,19 @@ void KatePluginGDBView::enableDebugActions(bool enable)
 
     m_ioView->enableInput(!enable && m_debugView->debuggerRunning());
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if ((m_lastExecLine > -1)) {
+        if (auto doc = m_kateApplication->findUrl(m_lastExecUrl)) {
+            if (enable) {
+                doc->setMarkDescription(KTextEditor::Document::Execution, i18n("Execution point"));
+                doc->setMarkIcon(KTextEditor::Document::Execution, QIcon::fromTheme(QStringLiteral("arrow-right")));
+                doc->addMark(m_lastExecLine, KTextEditor::Document::Execution);
+            } else {
+                doc->removeMark(m_lastExecLine, KTextEditor::Document::Execution);
+            }
+        }
+    }
+#else
     if ((m_lastExecLine > -1)) {
         KTextEditor::MarkInterfaceV2 *iface = qobject_cast<KTextEditor::MarkInterfaceV2 *>(m_kateApplication->findUrl(m_lastExecUrl));
 
@@ -523,6 +618,7 @@ void KatePluginGDBView::enableDebugActions(bool enable)
             }
         }
     }
+#endif
 }
 
 void KatePluginGDBView::programEnded()
@@ -549,6 +645,19 @@ void KatePluginGDBView::gdbEnded()
 
 void KatePluginGDBView::clearMarks()
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    const auto documents = m_kateApplication->documents();
+    for (KTextEditor::Document *doc : documents) {
+        const QHash<int, KTextEditor::Mark *> marks = doc->marks();
+        QHashIterator<int, KTextEditor::Mark *> i(marks);
+        while (i.hasNext()) {
+            i.next();
+            if ((i.value()->type == KTextEditor::Document::Execution) || (i.value()->type == KTextEditor::Document::BreakpointActive)) {
+                doc->removeMark(i.value()->line, i.value()->type);
+            }
+        }
+    }
+#else
     KTextEditor::MarkInterface *iface;
     const auto documents = m_kateApplication->documents();
     for (KTextEditor::Document *doc : documents) {
@@ -564,6 +673,7 @@ void KatePluginGDBView::clearMarks()
             }
         }
     }
+#endif
 }
 
 void KatePluginGDBView::slotSendCommand()
@@ -840,6 +950,13 @@ void KatePluginGDBView::handleEsc(QEvent *e)
 
 void KatePluginGDBView::enableBreakpointMarks(KTextEditor::Document *document)
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    if (document) {
+        document->setEditableMarks(document->editableMarks() | KTextEditor::Document::BreakpointActive);
+        document->setMarkDescription(KTextEditor::Document::BreakpointActive, i18n("Breakpoint"));
+        document->setMarkIcon(KTextEditor::Document::BreakpointActive, QIcon::fromTheme(QStringLiteral("media-record")));
+    }
+#else
     KTextEditor::MarkInterfaceV2 *iface = qobject_cast<KTextEditor::MarkInterfaceV2 *>(document);
 
     if (iface) {
@@ -847,6 +964,7 @@ void KatePluginGDBView::enableBreakpointMarks(KTextEditor::Document *document)
         iface->setMarkDescription(KTextEditor::MarkInterface::BreakpointActive, i18n("Breakpoint"));
         iface->setMarkIcon(KTextEditor::MarkInterface::BreakpointActive, QIcon::fromTheme(QStringLiteral("media-record")));
     }
+#endif
 }
 
 void KatePluginGDBView::displayMessage(const QString &msg, KTextEditor::Message::MessageType level)
@@ -866,4 +984,13 @@ void KatePluginGDBView::displayMessage(const QString &msg, KTextEditor::Message:
     kv->document()->postMessage(m_infoMessage);
 }
 
+QToolButton *KatePluginGDBView::createDebugButton(QAction *action)
+{
+    QToolButton *button = new QToolButton();
+    button->setDefaultAction(action);
+    button->setAutoRaise(true);
+    return button;
+}
+
+#include "moc_plugin_kategdb.cpp"
 #include "plugin_kategdb.moc"
