@@ -5,7 +5,6 @@
 */
 #pragma once
 
-#include <QSet>
 #include <QString>
 #include <QVector>
 
@@ -65,7 +64,7 @@ struct GitParsedStatus {
     QSet<QString> nonUniqueFileNames;
 };
 
-GitParsedStatus parseStatus(const QByteArray &raw, bool withNumStat, const QString &workingDir);
+GitParsedStatus parseStatus(const QByteArray &raw, const QString &workingDir);
 
 void parseDiffNumStat(QVector<GitUtils::StatusItem> &items, const QByteArray &raw);
 
