@@ -6,6 +6,7 @@
  */
 
 #include "kateprojectworker.h"
+#include "kateprojectindex.h"
 #include "kateprojectitem.h"
 
 #include <bytearraysplitter.h>
@@ -727,3 +728,5 @@ QVector<QString> KateProjectWorker::filesFromDirectory(QDir dir, bool recursive,
     }
     return files;
 }
+
+#include "moc_kateprojectworker.cpp"
