@@ -18,8 +18,10 @@
 #include <KPluginFactory>
 #include <KSharedConfig>
 #include <KTextEditor/Document>
-#include <KTextEditor/MovingInterface>
 #include <KTextEditor/View>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#include <KTextEditor/MovingInterface>
+#endif
 
 constexpr int numberOfColors = 5;
 
@@ -77,6 +79,13 @@ RainbowParenPluginView::RainbowParenPluginView(RainbowParenPlugin *plugin, KText
             rehighlight(view);
         }
     });
+    m_rehighlightTimer.setInterval(200);
+    m_rehighlightTimer.setSingleShot(true);
+    m_rehighlightTimer.callOnTimeout(this, [this] {
+        if (m_activeView) {
+            rehighlight(m_activeView);
+        }
+    });
 }
 
 static void getSavedRangesForDoc(std::vector<RainbowParenPluginView::SavedRanges> &ranges,
@@ -100,12 +109,18 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
 
     // disconnect and clear previous doc stuff
     if (m_activeView) {
-        disconnect(m_activeView, &KTextEditor::View::verticalScrollPositionChanged, this, &RainbowParenPluginView::rehighlight);
-        disconnect(m_activeView, &KTextEditor::View::textInserted, this, &RainbowParenPluginView::rehighlight);
+        disconnect(m_activeView, &KTextEditor::View::verticalScrollPositionChanged, this, &RainbowParenPluginView::onScrollChanged);
 
         auto doc = m_activeView->document();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        disconnect(doc, &KTextEditor::Document::aboutToDeleteMovingInterfaceContent, this, &RainbowParenPluginView::clearRanges);
+        disconnect(doc, &KTextEditor::Document::aboutToInvalidateMovingInterfaceContent, this, &RainbowParenPluginView::clearRanges);
+#else
         disconnect(doc, SIGNAL(aboutToDeleteMovingInterfaceContent(KTextEditor::Document *)), this, SLOT(clearRanges(KTextEditor::Document *)));
         disconnect(doc, SIGNAL(aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *)), this, SLOT(clearRanges(KTextEditor::Document *)));
+#endif
+        disconnect(doc, &KTextEditor::Document::textInserted, this, &RainbowParenPluginView::onTextInserted);
+        disconnect(doc, &KTextEditor::Document::textRemoved, this, &RainbowParenPluginView::onTextRemoved);
 
         // Remove if we already have this view's ranges saved
         clearSavedRangesForDoc(doc);
@@ -121,8 +136,17 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
         }
 
         // clear ranges for this doc if it gets closed
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        connect(doc, &KTextEditor::Document::aboutToDeleteMovingInterfaceContent, this, &RainbowParenPluginView::clearSavedRangesForDoc, Qt::UniqueConnection);
+        connect(doc,
+                &KTextEditor::Document::aboutToInvalidateMovingInterfaceContent,
+                this,
+                &RainbowParenPluginView::clearSavedRangesForDoc,
+                Qt::UniqueConnection);
+#else
         connect(doc, SIGNAL(aboutToDeleteMovingInterfaceContent(KTextEditor::Document *)), this, SLOT(clearSavedRangesForDoc(KTextEditor::Document *)));
         connect(doc, SIGNAL(aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *)), this, SLOT(clearSavedRangesForDoc(KTextEditor::Document *)));
+#endif
     }
 
     ranges.clear();
@@ -131,20 +155,67 @@ void RainbowParenPluginView::viewChanged(KTextEditor::View *view)
     // get any existing ranges for this view
     getSavedRangesForDoc(savedRanges, ranges, m_activeView->document());
 
-    connect(view, &KTextEditor::View::verticalScrollPositionChanged, this, &RainbowParenPluginView::rehighlight, Qt::UniqueConnection);
-    connect(view, &KTextEditor::View::textInserted, this, &RainbowParenPluginView::rehighlight, Qt::UniqueConnection);
+    connect(view, &KTextEditor::View::verticalScrollPositionChanged, this, &RainbowParenPluginView::onScrollChanged, Qt::UniqueConnection);
 
     auto doc = m_activeView->document();
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    connect(doc, &KTextEditor::Document::aboutToDeleteMovingInterfaceContent, this, &RainbowParenPluginView::clearRanges, Qt::UniqueConnection);
+    connect(doc, &KTextEditor::Document::aboutToInvalidateMovingInterfaceContent, this, &RainbowParenPluginView::clearRanges, Qt::UniqueConnection);
+#else
     connect(doc, SIGNAL(aboutToDeleteMovingInterfaceContent(KTextEditor::Document *)), this, SLOT(clearRanges(KTextEditor::Document *)), Qt::UniqueConnection);
     connect(doc,
             SIGNAL(aboutToInvalidateMovingInterfaceContent(KTextEditor::Document *)),
             this,
             SLOT(clearRanges(KTextEditor::Document *)),
             Qt::UniqueConnection);
+#endif
+    connect(doc, &KTextEditor::Document::textInserted, this, &RainbowParenPluginView::onTextInserted, Qt::UniqueConnection);
+    connect(doc, &KTextEditor::Document::textRemoved, this, &RainbowParenPluginView::onTextRemoved, Qt::UniqueConnection);
 
     rehighlight(m_activeView);
 }
 
+static void onTextChanged(RainbowParenPluginView *p, const QString &text)
+{
+    auto isBracket = [](QChar c) {
+        return c == QLatin1Char('{') || c == QLatin1Char('(') || c == QLatin1Char(')') || c == QLatin1Char('}') || c == QLatin1Char('[')
+            || c == QLatin1Char(']');
+    };
+    if (text.size() > 100) {
+        p->requestRehighlight();
+        return;
+    }
+
+    for (auto c : text) {
+        if (isBracket(c)) {
+            p->requestRehighlight();
+            break;
+        }
+    }
+}
+
+void RainbowParenPluginView::onTextInserted(KTextEditor::Document *, KTextEditor::Cursor, const QString &text)
+{
+    onTextChanged(this, text);
+}
+
+void RainbowParenPluginView::onTextRemoved(KTextEditor::Document *, KTextEditor::Range, const QString &text)
+{
+    onTextChanged(this, text);
+}
+
+void RainbowParenPluginView::onScrollChanged()
+{
+    requestRehighlight(50);
+}
+
+void RainbowParenPluginView::requestRehighlight(int delay)
+{
+    if (!m_rehighlightTimer.isActive()) {
+        m_rehighlightTimer.start(delay);
+    }
+}
+
 void RainbowParenPluginView::clearRanges(KTextEditor::Document *)
 {
     ranges.clear();
@@ -160,6 +231,8 @@ void RainbowParenPluginView::clearSavedRangesForDoc(KTextEditor::Document *doc)
     }
 }
 
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+
 static bool isComment(KTextEditor::Document *doc, int line, int col)
 {
     return doc->defaultStyleAt({line, col}) == KTextEditor::DefaultStyle::dsComment;
@@ -171,11 +244,26 @@ static bool isString(KTextEditor::Document *doc, int line, int col)
     return defStyle == KTextEditor::DefaultStyle::dsChar || defStyle == KTextEditor::DefaultStyle::dsString;
 }
 
+#else
+
+static bool isComment(KTextEditor::Document *doc, int line, int col)
+{
+    return doc->defaultStyleAt({line, col}) == KSyntaxHighlighting::Theme::TextStyle::Comment;
+}
+
+static bool isString(KTextEditor::Document *doc, int line, int col)
+{
+    const auto defStyle = doc->defaultStyleAt({line, col});
+    return defStyle == KSyntaxHighlighting::Theme::TextStyle::Char || defStyle == KSyntaxHighlighting::Theme::TextStyle::String;
+}
+
+#endif
+
 using ColoredBracket = std::unique_ptr<KTextEditor::MovingRange>;
 using ColoredBracketPair = std::pair<std::unique_ptr<KTextEditor::MovingRange>, std::unique_ptr<KTextEditor::MovingRange>>;
 
 /**
- * Helper function to find if we have @p r already in oldRanges so we
+ * Helper function to find if we have @p open and @p close already in oldRanges so we
  * can reuse it
  */
 static ColoredBracketPair existingColoredBracketForPos(std::vector<ColoredBracket> &oldRanges, KTextEditor::Cursor open, KTextEditor::Cursor close)
@@ -310,7 +398,9 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
         }
     }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     auto miface = qobject_cast<KTextEditor::MovingInterface *>(doc);
+#endif
 
     // We reuse ranges completely if there was no change but the user
     // was only scrolling. This allows the colors to stay somewhat stable
@@ -374,12 +464,20 @@ void RainbowParenPluginView::rehighlight(KTextEditor::View *view)
             continue;
         }
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        std::unique_ptr<KTextEditor::MovingRange> r(doc->newMovingRange({p.opener, cur1}));
+#else
         std::unique_ptr<KTextEditor::MovingRange> r(miface->newMovingRange({p.opener, cur1}));
+#endif
         r->setAttribute(attrs[color % numberOfColors]);
 
         auto cur2 = p.closer;
         cur2.setColumn(cur2.column() + 1);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        std::unique_ptr<KTextEditor::MovingRange> r2(doc->newMovingRange({p.closer, cur2}));
+#else
         std::unique_ptr<KTextEditor::MovingRange> r2(miface->newMovingRange({p.closer, cur2}));
+#endif
         r2->setAttribute(attrs[color % numberOfColors]);
 
         ranges.push_back(std::move(r));
@@ -429,9 +527,8 @@ QString RainbowParenConfigPage::fullName() const
 
 static QIcon ColoredBracketsIcon(const QWidget *_this)
 {
-    const auto icon = QIcon::fromTheme(QStringLiteral("code-context"));
-    auto p = icon.pixmap(QSize(16, 16));
-    if (p.isNull()) {
+    QPixmap p;
+    {
         QRect r(QPoint(0, 0), _this->devicePixelRatioF() * QSize(16, 16));
         QPixmap pix(r.size());
         pix.fill(Qt::transparent);
@@ -485,4 +582,5 @@ void RainbowParenConfigPage::defaults()
 {
 }
 
+#include "moc_rainbowparens_plugin.cpp"
 #include "rainbowparens_plugin.moc"
